<?php

namespace spec\Collector\Core\HealthCheck;

use Collector\Core\HealthCheck;
use Collector\Infrastructure\HealthCheckInMemory;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class ServiceSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(new HealthCheckInMemory());
    }

    function it_adds_a_new_helthcheck()
    {
        $current_health = $this->aHealthCheck();

        $this->register($current_health);
        $this->display()->shouldContains($current_health);
    }

    function it_should_get_healthchecks_collection()
    {
        $this->display()->shouldHaveType('Collector\Core\HealthCheck\Collection');
    }

    function it_should_contains_all_systems_in_collection()
    {
        $this->register($this->aHealthCheck());
        $this->register($this->aHealthCheck());
        $this->register($this->aHealthCheck());

        $this->display()->shouldHaveCount(3);
    }

    function it_should_retrieve_healthchecks_collection()
    {
        $this->retrieve('anyid')->shouldHaveType('Collector\Core\HealthCheck\Collection');
    }

    function it_should_retrieve_a_system_health()
    {
        $aHealthCheck = $this->aHealthCheck();

        $this->register($aHealthCheck);
        $this->register($this->aHealthCheck());
        $this->register($this->aHealthCheck());

        $this->retrieve($aHealthCheck->id())->shouldHaveCount(1);
        $this->retrieve($aHealthCheck->id())->shouldContains($aHealthCheck);
    }

    private function aHealthCheck()
    {
        return new HealthCheck(uniqid('', true), 1.2, 128, 256.7);
    }

    public function getMatchers()
    {
        return [
            'contains' => function(\ArrayIterator $collection, $item) {
                return in_array($item, $collection->getArrayCopy());
            }
        ];
    }


}
