<?php

namespace spec\Collector\Core\System;

use Collector\Core\System;
use Collector\Infrastructure\SystemInMemory;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class ServiceSpec extends ObjectBehavior
{
    function let()
    {
        $this->beConstructedWith(new SystemInMemory());
    }

    function it_should_register_new_system()
    {
        $aSystem = $this->aSystem();
        $this->create($aSystem);
    }

    function it_should_retrieve_system_by_id()
    {
        $aSystem = $this->aSystem();
        $this->create($aSystem);
        $this->retrieve($aSystem->id())->shouldBe($aSystem);
    }

    function it_should_return_null_if_does_not_exist_system_with_given_id()
    {
        $this->retrieve('doesnotexist')->shouldBeNull();
    }

    function it_should_replace_a_whole_system_by_new_one()
    {
        $old_system = new System('anyid', 'anyhostname', '191.168.1.10', 2, 1024, 2048, 'linux');
        $new_system = new System($old_system->id(), 'anyhostname', '191.168.1.10', 4, 2048, 4096, 'linux');

        $this->create($old_system);
        $this->replace($old_system->id(), $new_system);
        $this->retrieve($old_system->id())->shouldBeLike($new_system);
    }

    function it_should_create_new_system_if_replace_did_not_match_any_system()
    {
        $aSystem = $this->aSystem();
        $this->replace($aSystem->id(), $aSystem);
        $this->retrieve($aSystem->id())->shouldBeLike($aSystem);
    }

    function it_should_throw_an_error_if_old_system_and_new_one_have_different_ids()
    {
        $old_system = $this->aSystem();
        $new_system = $this->aSystem();
        $this->create($old_system);

        $this->shouldThrow('\DomainException')->duringReplace($old_system->id(), $new_system);
    }

    function it_should_delete_system_by_id()
    {
        $aSystem = $this->aSystem();
        $this->create($aSystem);
        $this->delete($aSystem->id());
        $this->retrieve($aSystem->id())->shouldBe(null);
    }

    function it_should_throw_an_error_if_tries_to_delete_unexistant_system()
    {
        $this->shouldThrow('\InvalidArgumentException')->duringDelete('nonexist');
    }

    function it_should_get_system_collection()
    {
        $this->get()->shouldHaveType('Collector\Core\System\Collection');
    }

    function it_should_contains_all_systems_in_collection()
    {
        $this->create($this->aSystem());
        $this->create($this->aSystem());
        $this->create($this->aSystem());

        $this->get()->shouldHaveCount(3);
    }

    private function aSystem()
    {
        return new System(
            uniqid('', true),
            'anyhostname',
            '191.168.1.10',
            mt_rand(1, 16),
            mt_rand(1024, 2048),
            mt_rand(1024, 2048),
            'linux'
        );
    }
}
