<?php

namespace spec\Collector\Core;

use Collector\Core\System;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class SystemSpec extends ObjectBehavior
{
    const ANY_HOSTNAME = "Saturno";
    const ANY_IP = "192.168.1.10";
    const ANY_OPERATING_SYSTEM = "Linux";
    const ANY_HDD_CAPACITY = 1048576.0;
    const ANY_RAM_CAPACITY = 2048.0;
    const ANY_CPU_NUMBER = 4;

    function let()
    {
        $this->makeThis();
    }

    function it_should_expose_its_id()
    {
        $this->id()->shouldBe("id");
    }

    function it_should_expose_hostname()
    {
        $this->hostname()->shouldBe('Saturno');
    }

    function it_should_expose_ip()
    {
        $this->ip()->shouldBe('192.168.1.10');
    }

    function it_should_expose_empty_ip_if_not_valid()
    {
        $this->beConstructedWith(
            'anyid',
            self::ANY_HOSTNAME,
            'NOT_VALID_IP',
            self::ANY_CPU_NUMBER,
            self::ANY_RAM_CAPACITY,
            self::ANY_HDD_CAPACITY,
            self::ANY_OPERATING_SYSTEM
        );
        $this->ip()->shouldBe('0.0.0.0');
    }

    function it_should_expose_number_of_cpus()
    {
        $this->numberOfCpus()->shouldBe(self::ANY_CPU_NUMBER);
    }

    function it_should_expose_ram_capacity()
    {
        $this->ramCapacity()->shouldBe(self::ANY_RAM_CAPACITY);
    }

    function it_should_expose_hdd_capacity()
    {
        $this->hddCapacity()->shouldBe(self::ANY_HDD_CAPACITY);
    }

    function it_should_expose_operating_system()
    {
        $this->operatingSystem()->shouldBe(self::ANY_OPERATING_SYSTEM);
    }

    function it_should_be_comparable()
    {
        $this->beConstructedWith(
            "any_id",
            'anyhostname',
            self::ANY_IP,
            self::ANY_CPU_NUMBER,
            self::ANY_RAM_CAPACITY,
            self::ANY_HDD_CAPACITY,
            self::ANY_OPERATING_SYSTEM
        );
        $this->equalsTo($this->aSystem('other_id'))->shouldReturn(false);
        $this->equalsTo($this->aSystem('any_id'))->shouldReturn(true);
    }

    function it_should_populate_created_date()
    {
        $this->createdAt()->shouldHaveType('\DateTimeImmutable');
    }

    private function makeThis()
    {
        $this->beConstructedWith(
            "id",
            self::ANY_HOSTNAME,
            self::ANY_IP,
            self::ANY_CPU_NUMBER,
            self::ANY_RAM_CAPACITY,
            self::ANY_HDD_CAPACITY,
            self::ANY_OPERATING_SYSTEM
        );
    }


    private function aSystem($id)
    {
        return new System(
            $id ?: uniqid('', true),
            self::ANY_HOSTNAME,
            self::ANY_IP,
            mt_rand(1, 16),
            mt_rand(1024, 2048),
            mt_rand(1024, 2048),
            'linux'
        );
    }
}
