<?php

namespace Collector\Core;

use Rych\ByteSize\ByteSize;
use Collector\Core\System\Id;

class System implements \JsonSerializable
{
    private $id;
    private $hostname;
    private $ip;
    private $cpus;
    private $ram;
    private $hdd;
    private $operating_system;
    private $created_at;

    public function __construct($id, $hostname, $ip, $cpus, $ram, $hdd, $operating_system)
    {
        $this->id = $id instanceof Id ? $id : new Id($id);
        $this->hostname = (string) $hostname;
        $this->ip = $this->ipAsLong($ip) ? (string) $ip : long2ip($ip);
        $this->cpus = (int) $cpus;
        $this->ram = (float) $ram;
        $this->hdd = (float) $hdd;
        $this->operating_system = (string) $operating_system;
        $this->created_at = new \DateTimeImmutable('now', new \DateTimeZone('UTC'));
    }

    public function id()
    {
        return $this->id->getId();
    }

    public function hostname()
    {
        return $this->hostname;
    }

    public function ip()
    {
        return $this->ip;
    }

    public function numberOfCpus()
    {
        return $this->cpus;
    }

    public function ramCapacity()
    {
        return $this->ram;
    }

    public function hddCapacity()
    {
        return $this->hdd;
    }

    public function operatingSystem()
    {
        return $this->operating_system;
    }

    public function equalsTo(System $other)
    {
        return $this->id->equalsTo($other->id);
    }

    public function createdAt()
    {
        return $this->created_at;
    }

    public function toArray()
    {
        return [
            'id' => $this->id(),
            'hostname' => $this->hostname(),
            'ip' => $this->ipAsLong($this->ip()),
            'created_at' => $this->createdAt(),
            'cpus' => $this->numberOfCpus(),
            'ram_capacity' => $this->ramCapacity(),
            'hdd_capacity' => $this->hddCapacity(),
            'operating_system' => $this->operatingSystem(),
        ];
    }

    /**
     * (PHP 5 &gt;= 5.4.0)<br/>
     * Specify data which should be serialized to JSON
     * @link http://php.net/manual/en/jsonserializable.jsonserialize.php
     * @return mixed data which can be serialized by <b>json_encode</b>,
     * which is a value of any type other than a resource.
     */
    public function jsonSerialize()
    {
        // Yeah, this is not the best place to set up a dependency.
        $bytes_formatter = new ByteSize();
        return [
            'id' => $this->id(),
            'hostname' => $this->hostname(),
            'ip' => $this->ip(),
            'cpus' => $this->numberOfCpus(),
            'ram' => $bytes_formatter->formatBinary($this->ramCapacity(), 2),
            'hdd' => $bytes_formatter->formatBinary($this->hddCapacity(), 2),
            'os' => $this->operatingSystem(),
        ];
    }

    private function ipAsLong($ip)
    {
        return ip2long($ip) ?: 0;
    }
}
