<?php

namespace Collector\Infrastructure;

use Silex\Application;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Collector\Core\System;
use Collector\Core\HealthCheck;

class Controller
{
    public function indexAction(Application $app, Request $request)
    {
        $base_url = $request->getSchemeAndHttpHost();

        return $app->json([
            'systems' => $base_url . '/systems',
        ]);
    }


    public function listSystemsAction(Application $app)
    {
        $systems = $app['system.service']->get();
        $system_to_display = [];
        foreach ($systems as $system) {
            $system_to_display[] = array_merge(
                $system->jsonSerialize(),
                array('_link' => $app['url_generator']->generate('systems_by_id', ['id' => $system->id()], true))
            );
        }
        return $app->json($system_to_display);
    }


    public function createSystemAction(Application $app, Request $request){
        $system_to_persist = json_decode($request->getContent());

        try {
            $system = new System(
                $system_to_persist->id,
                $system_to_persist->hostname,
                $system_to_persist->ip,
                $system_to_persist->cpus,
                $system_to_persist->ram,
                $system_to_persist->hdd,
                $system_to_persist->os
            );
            $app['system.service']->create($system);
        } catch (\DomainException $e) {
            return $app->json(['errors' => ['id' => ['cannot be null']]], 409);
        }

        return $app->json($system, 201);
    }


    public function deleteSystemAction(Application $app, $id)
    {
        $response_code = 204;

        try {
            $app['system.service']->delete($id);
        } catch (\InvalidArgumentException $e)
        {
            $response_code = 404;
        }

        return new Response('', $response_code, ['Content-Type' => 'application/json']);
    }


    public function showSystemAction(Application $app, $id)
    {
        $system = $app['system.service']->retrieve($id);
        if (!empty($system)) {
            $system = array_merge(
                $system->jsonSerialize(),
                array('_link' => $app['url_generator']->generate('healths', ['id' => $system->id()], true))
            );
        }
        return $app->json($system, ($system ? 200 : 404));
    }


    public function replaceSystemAction(Application $app, Request $request, $id)
    {
        $response_code = 200;
        $system_to_update = json_decode($request->getContent());
        $system_to_update = new System(
            $id,
            $system_to_update->hostname,
            $system_to_update->ip,
            $system_to_update->cpus,
            $system_to_update->ram,
            $system_to_update->hdd,
            $system_to_update->os
        );

        try {
            $app['system.service']->replace($id, $system_to_update);
        } catch (\DomainException $e) {
            $response_code = 409;
        }

        return $app->json($system_to_update, $response_code);
    }


    public function listHealthsAction(Application $app, $id)
    {
        return $app->json($app['healthcheck.service']->retrieve($id));
    }


    public function createHealthAction(Application $app, Request $request, $id)
    {
        $measure = json_decode($request->getContent());
        $measure = new HealthCheck(
            $id,
            $measure->cpu_load,
            $measure->used_ram,
            $measure->used_hdd
        );
        $app['healthcheck.service']->register($measure);

        return $app->json($measure, 201);
    }
}