<?php

namespace Collector\Infrastructure;

use Collector\Core\HealthCheck;
use Collector\Core\HealthCheck\Repository;
use Collector\Core\System\Id;

class HealthCheckMongodb implements Repository
{
    const DATABASE_NAME = 'grid';
    const COLLECTION_NAME = 'healthcheck';

    /**
     * 40 * 30 segs (hearth beat) = 1200 segs => 20 mins of hearth beats.
     */
    const MAX_RESULTS = 40;

    private $db_connection;
    private $collection;

    public function __construct(\MongoClient $db_connection)
    {
        $this->db_connection = $db_connection;
        $this->collection = $this->db_connection->selectCollection(self::DATABASE_NAME, self::COLLECTION_NAME);
    }

    public function add(HealthCheck $current)
    {
        $this->collection->save($current->toArray());
    }

    public function findAll()
    {
        return $this->doFind();
    }

    public function findById(Id $system_id)
    {
        return $this->doFind(['id' => (string) $system_id]);
    }

    private function doFind(Array $query = [])
    {
        return $this->createAndPopuleCollection(
            $this->collection->find($query)->limit(self::MAX_RESULTS)
        );
    }

    private function createAndPopuleCollection($healths)
    {
        $healthcheck_collection = new HealthCheck\Collection();

        foreach ($healths as $health) {
            $healthcheck_collection->append(
                $this->makeAHealthCheckInstance($health)
            );
        }

        return $healthcheck_collection;
    }

    private function makeAHealthCheckInstance($health)
    {
        return new HealthCheck(
            $health['id'],
            $health['cpu_load'],
            $health['used_ram'],
            $health['used_hdd'],
            $health['created_at']
        );
    }
}