<?php

namespace Collector\Infrastructure;

use Collector\Core\System;
use Collector\Core\System\Id;
use Collector\Core\System\Repository;

class SystemMongodb implements Repository
{
    const DATABASE_NAME = 'grid';
    const COLLECTION_NAME = 'system';

    private $db_connection;
    private $collection;

    public function __construct(\MongoClient $db_connection)
    {
        $this->db_connection = $db_connection;
        $this->collection = $this->db_connection->selectCollection(self::DATABASE_NAME, self::COLLECTION_NAME);
    }

    public function add(System $system)
    {
        $this->collection->insert($this->transformSystemToMongoLike($system));
    }

    public function findAll()
    {
        return $this->doFind();
    }

    public function findById(Id $system_id)
    {
        return $this->doFind(['_id' => $system_id->getId()])->current();
    }

    public function update(Id $system_id, System $new_system)
    {
        $this->collection->save($this->transformSystemToMongoLike($new_system));
    }

    public function delete(Id $system_id)
    {
        $this->collection->remove(['_id' => $system_id->getId()]);
    }

    private function transformSystemToMongoLike(System $system)
    {
        $system_to_insert = $system->toArray();
        $system_to_insert['_id'] = $system_to_insert['id'];
        unset($system_to_insert['id']);

        return $system_to_insert;
    }

    private function doFind(Array $query = [])
    {
        return $this->createAndPopuleCollection(
            $this->collection->find($query)
        );
    }

    private function createAndPopuleCollection($systems)
    {
        $system_collection = new System\Collection();

        foreach ($systems as $system) {
            $system_collection->append(
                $this->makeASystemInstance($system)
            );
        }

        return $system_collection;
    }

    private function makeASystemInstance($system)
    {
        return new System(
            $system['_id'],
            $system['hostname'],
            long2ip($system['ip']),
            $system['cpus'],
            $system['ram_capacity'],
            $system['hdd_capacity'],
            $system['operating_system']
        );
    }
}