<?php

namespace GridAdmin;

use GuzzleHttp\Pool;
use Silex\Application;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

class Controller
{
    public function indexAction(Application $app)
    {
        $systems = $app['http_client']->get(
            $app['information_collector.endpoints']['systems'],
            $app['http_client.default_config']
        )->json();
        $health = $this->getSystemsHealth($app, $systems);
        $systems_aggregated = [];
        $index = 0;

        foreach ($systems as $system) {
            $systems_aggregated[] = new SystemAggregate($system, $health[$index++]);
        }

        return $app['twig']->render('@frontend/homepage.twig', ['systems' => $systems_aggregated]);
    }

    public function deleteSystemAction(Application $app, $id)
    {
        $system = ['id' => $id];
        $health = $this->getSystemsHealth($app, [$system]);
        $aggregated = new SystemAggregate($system, $health[0]);
        $response_message = '';
        $response_code = 204;

        if ($aggregated->is_up()) {
            $response_message = json_encode(['message' => 'System is up, you can not remove it.']);
            $response_code = 409;
        } else {
            try {
                $app['http_client']->delete(
                    [
                        $app['information_collector.endpoints']['system_by_id'],
                        ['systems_id' => $id]
                    ],
                    $app['http_client.default_config']
                );
            } catch (BadResponseException $e) {
                $response_code = 500;
            }
        }

        return new Response($response_message, $response_code, ['Content-Type' => 'application/json']);
    }

    public function downloadsAction(Application $app)
    {
        return $app['twig']->render('@frontend/downloads.twig');
    }

    public function rolesAction(Application $app, Request $request)
    {
        $post = json_decode($request->getContent());
        $user_id = $post->user_id;
        $new_roles = $post->new_roles;
        $userManager = $app['user.manager'];

        $user = $userManager->getUser($user_id);
        if (!$user) {
            throw new NotFoundHttpException('No user was found with that ID.');
        }

        $user->setRoles($new_roles);
        $errors = $userManager->validate($user);

        if (empty($errors)) {
            $userManager->update($user);
            return new Response('');
        }

        return new JsonResponse($errors, 500);
    }

    private function getSystemsHealth(Application $app, $systems)
    {
        $requests = array_map(
            function ($system) use ($app) {
                return $app['http_client']->createRequest(
                    'GET',
                    [
                        $app['information_collector.endpoints']['healths'],
                        ['systems_id' => $system['id']]
                    ],
                    $app['http_client.default_config']
                );
            },
            $systems
        );

        return Pool::batch($app['http_client'], $requests);
    }
}