<?php

namespace GridAdmin;

class SystemAggregate
{
    /**
     * How many seconds to wait until set a System as down.
     */
    const UP_AND_RUNNING_THRESHOLD = 1800;

    private $id;
    private $hostname;
    private $ip;
    private $cpus;
    private $ram;
    private $hdd;
    private $os;
    private $is_up;
    private $last_healthcheck;

    public function __construct($system, $healthcheck_list)
    {
        $this->id = $this->getValueOrDefault($system, 'id', '');
        $this->hostname = $this->getValueOrDefault($system, 'hostname', '');
        $this->ip = $this->getValueOrDefault($system, 'ip', '0.0.0.0');
        $this->cpus = $this->getValueOrDefault($system, 'cpus', 0);
        $this->ram = $this->getValueOrDefault($system, 'ram', 0);
        $this->hdd = $this->getValueOrDefault($system, 'hdd', 0);
        $this->os = $this->getValueOrDefault($system, 'os', '');

        $this->last_healthcheck = $this->calculateLastHealthcheck($healthcheck_list->json());
        $this->is_up = $this->checkIfSystemIsUp($this->last_healthcheck);
    }

    public function id()
    {
        return $this->id;
    }

    public function hostname()
    {
        return $this->hostname;
    }

    public function ip()
    {
        return $this->ip;
    }

    public function cpus()
    {
        return $this->cpus;
    }

    public function ram()
    {
        return $this->ram;
    }

    public function hdd()
    {
        return $this->hdd;
    }

    public function os()
    {
        return $this->os;
    }

    public function is_up()
    {
        return $this->is_up;
    }

    public function last_healthcheck()
    {
        return $this->last_healthcheck instanceof \DateTimeInterface
            ? $this->last_healthcheck->getTimestamp()
            : '';
    }

    private function getValueOrDefault($system, $key, $default)
    {
        return isset($system[$key]) ? $system[$key] : $default;
    }

    private function checkIfSystemIsUp($last_healtcheck)
    {
        if (empty($last_healtcheck)) {
            return false;
        }

        $now = new \DateTimeImmutable('now', new \DateTimeZone('UTC'));

        return $now <= $last_healtcheck->add(new \DateInterval('PT' . self::UP_AND_RUNNING_THRESHOLD . 'S'));
    }

    private function makeDateFromIsoFormat($date_in_iso)
    {
        return \DateTimeImmutable::createFromFormat(\DateTime::ISO8601, $date_in_iso, new \DateTimeZone('UTC'));
    }

    private function calculateLastHealthcheck($healtcheck_list)
    {
        if (!(count($healtcheck_list) && isset($healtcheck_list[0]) && isset($healtcheck_list[0]['created_at']))) {
            return null;
        }

        return $this->makeDateFromIsoFormat($healtcheck_list[0]['created_at']);
    }
}